/* Icons
 ****************************************************************************/
/* Add some margin to icon images like profil, or logout. (these are not the
 * bootstrap icons) */
img.icon {
    margin: 0 5px;
}

/* Bootstrap overwrites
 ****************************************************************************/
/* Hide carret in dropup menus as they are displayed in up direction which
 * does not fit for the administrationial menu. So hide them all.
 * TODO: Display caret but in down direction. */
.navbar .nav > li.dropup > .dropdown-menu::after, 
.navbar .nav > li.dropup > .dropdown-menu::before {
  display:none;
}

/* Overwrite the following CSS to change the color of the navigation bar
.navbar-default {
  background-color: #f8f8f8;
  border-color: #e7e7e7;
  background-image: linear-gradient(to bottom,#f8f8f8 0,#e7e7e7 100%);
}
*/


/* Render the top border of the navigation bar to have a better visual
 * separatin to the logo above the navigation. */
.navbar-static-top {
  border-width: 1px 0px;
}

/* Reduce margins to make the content under the page-header more compact. */
.page-header {
  margin: -10px 0px 3px 0px;
}
.page-header h1 {
  margin: 0px 0;
  clear: left;
}

#footer  {
  margin: 5px 0;
  padding: 17px 0px;
}

#footer a {
  color: rgb(119, 119, 119);
  text-decoration: none;
}

/* BS3 seems to make the th 2px hight which i do not like. So set it back to
 * 1px */
.table-bordered > thead > tr > th, .table-bordered > thead > tr > td {
    border-bottom-width: 1px;
}
.table thead > tr > th {

    vertical-align: bottom;
    border-bottom: 1px solid rgb(221, 221, 221);
}

.modal-lg {
  width: 80%;
}

/* pagination margin must be 0 to match ringo layout */
nav ul.pagination {
  margin: 0;
}

/* Media depended affix CSS */
@media(max-width:767px){
  .formbar-outline.affix {
    position: static;
    width: 100%;
    top: 0;
    z-index:10;
  }
}

@media(min-width:768px){
  .formbar-outline.affix {
    position: fixed;
    top: 0;
    z-index:10;
    width: 157.5px;
  }
}
@media(min-width:992px){
  .formbar-outline.affix {
    width: 212.5px;
  }
}
@media(min-width:1200px){
  .formbar-outline.affix {
    width: 262.5px;
  }
}
/* affix default (bootstrap overwrite) */
.affix{
  position:static;
}

/* dont show link href in print layout
 * See http://drupal.stackexchange.com/questions/59900/how-to-get-rid-of-added-urls-when-printing-a-bootstrap-themed-page */
@media print {
  a[href]:after {
    content: none;
  }
}
