#spinner {
  width: 200px;
  position: fixed;
  top: 50%;
  left: 42%;
  z-index: 200000000000000000;
}    

#spinnerContainer .modal-dialog {
  width: 200px;
  top: 30%;
}
#spinnerContainer .modal-content {
  height: 200px;
}
#spinnerContainer .modal-body {
  height: 200px;
}
/* Context menu
 * *************************************************************************/
/* Chane positon to align the menu with the page header on the left side. */
.context-menu {
  position: relative;
  top: 2px;
}

/***************************************************************************
 *                               Main panes                                *
 ***************************************************************************/
.main-pane {
    display: none
}
.main-pane.active {
  display: inline;
}

/****************************************************************************
 *                               Header/Logo                                *
 ****************************************************************************/
#logo a {
  color: rgb(119, 119, 119);
  font-size: small;
  text-decoration: none;
}

.navbar-static-top {
  margin-bottom: 0px;
}

/**********************************************************************
 *                             Logininfo                              *
 **********************************************************************/

#logininfo {
    text-align: right;
    font-size: 0.85em;
}

#logininfowarning {
    margin-top: 15px;
    margin-bottom: 15px;
}

/****************************************************************************
 *                               Sessiontimer                               *
 ****************************************************************************/

#sessiontimer {
  width: 100px;
  position: relative;
  top: 10px;
}

#sessiontimer input {
  background-color: white;
}

#sessiontimer div.input-group-addon {
  cursor: pointer;
}

/****************************************************************************
 *                                Breadcrumb                                *
 ****************************************************************************/

#breadcrumb {
  margin-bottom: 20px;
}

#breadcrumb ol {
  padding: 5px 15px;
  font-size: smaller;
  margin-bottom: 0px;
}

/* Status message
 * *************************************************************************/
/* Push the status message a littlebit down from the page header. */
#status-messages {
  margin-top: 0px;
  width: 100%;
  position: absolute;
  z-index: 9999999;
  text-align: center;
}

#status-messages .alert {
  padding: 5px 15px;
  margin-bottom: 0px;
  border-top: 0px solid transparent;
  border-top-right-radius: 0px;
  border-top-left-radius: 0px;
}

/* Overview tables
 * *************************************************************************/
/* Change the mouse cursor to a pointer when hovering the rows of the
 * overviewtable to indicate that they are clickable. */
.table {
    margin-bottom: 10px;
}

.table td a.link {
  text-decoration: none;
  color: black;
}

.table tbody td.link {
    cursor:pointer
}

.table td a.filter {
  border-top: 1px dotted;
  cursor: zoom-in;
}

table.content-shorten {
  display: table;  
  border-collapse: collapse;
}

table.content-shorten td {
  max-width: 0;
  white-space: nowrap;
  overflow: hidden;
  text-overflow: ellipsis;
}

#pagination-size-selector {
    margin-right: 20px;
}

#pagination-size-selector select {
  width: auto;
  display: inline;
}

 /* Datatables
 **********************************************************************/

/* Overwrite padding in th to be aligend with the rest of the table.
 * This change was neeed after enabling the sorting icons. */
table.dataTable thead th {
    padding: 10px 10px;
}

/* Do not show the sorting indicator for the checkboxrow */
table.dataTable thead th.checkboxrow {
    background-image: none;
}

div.dataTables_length {
    float: right;
    padding: 10px;
}

div.dataTables_length select {
    display: inline;
    width: 80px;
}

.dataTables_filter input[type=search] {
    margin-left: 5px;
    margin-right: 10px;
    margin-bottom: 0px;
    width: 200px;
    display: inline;
}


.search-filters input[type=checkbox] {
  position: relative;
  margin-right: 3px;
  top: -2px;
}


.search-filters div.searchfilter {
  margin-right: 15px;
}

div.dataTables_filter{
    display: inline-block;
}

div.dataTables_info {
    position: relative;
    top: -8px;
    font-size: 0.9em;
}

div.dataTables_paginate {
    position: relative;
    top: -10px;
    font-size: 0.9em;
    float: right;
}

a.paginate_button,
a.paginate_active {
    padding: 2px 2px;
    margin-left: 2px;
    cursor: pointer;
    *cursor: hand;
}
 
a.paginate_active {
    background-color: transparent;
    font-weight: bold;
}
 
.paging_full_numbers a:active {
    outline: none
}
.paging_full_numbers a:hover {
    text-decoration: none;
}
 
div.dataTables_paginate span>a {
    width: 15px;
    text-align: center;
}

/* Self clearing - */
div.dataTables_wrapper:after {
    content: ".";
    display: block;
    clear: both;
    visibility: hidden;
    line-height: 0;
    height: 0;
}
html[xmlns] .dataTables_wrapper { display: block; }
* html .dataTables_wrapper { height: 1%; }
 
.table .table-toolbar {
    background-color: #F5F5F5;
}

.table .actions {
    text-align: right;
    width: 100px;
}

/* Search
 * *************************************************************************/
/* Change the mouse cursor to a pointer when hovering the rows of the
 * overviewtable to indicate that they are clickable. */
.search-widget {
  margin-bottom: 10px;
  padding: 7px;
  border: 1px dashed;
  border-color: #ddd;
  background: #f7f7f7;
  border-radius: 4px;
}
.search-widget form {
  margin-bottom: 0px;
}
/* Overwrite the margin of the button (was 15px) */
.search-widget button {
    margin-right: 0px;
}
ul.dropdown-menu li table td a {
    padding: 10px 15px;
    color: black;
    text-decoration: None;
}
ul.dropdown-menu li table {
    margin-bottom: 0px;
}
ul.dropdown-menu li table td {
    vertical-align: middle;
}

/* Statefield */

/* Hide infos on the resulting state on default. */
.result-state {
  display: none
}

/* Custom formbar forms */
.formbar-linkfield {
  margin-top: 1.45em;
  white-space: normal;
}
